/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.FloatingLicenseUsageView");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");
dojo.require("dojo.date");
dojo.require("dojo.date.stamp");
dojo.require("dojox.collections.ArrayList");
dojo.require("dojo.cookie");

dojo.require("com.ibm.team.repository.web.ui.internal.admin.ItemList");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.licensing");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.util");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.FloatingLicenseTypeCombo");
dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "FloatingLicenseUsageViewMessages");

var ArrayList = dojox.collections.ArrayList;
var util = com.ibm.team.repository.web.ui.internal.admin.util;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var LicenseClient = com.ibm.team.repository.web.client.internal.licensing;
var FloatingLicenseTypeCombo = com.ibm.team.repository.web.ui.internal.admin.server.FloatingLicenseTypeCombo;
var formatTimeFromMilliseconds = com.ibm.team.repository.web.ui.internal.admin.util.formatTimeFromMilliseconds;
var replaceNewLineWithPaddedBreaks = com.ibm.team.repository.web.ui.internal.admin.util.replaceNewLineWithPaddedBreaks;
var FLOATING_LICENSE_SERVICE_URL = "/service/com.ibm.team.repository.service.internal.license.IFloatingLicenseService/licenses";
var FLOATING_LICENSE_SERVICE_URL_METHOD = "/in-use";

function isNode(o){
	return o && o.nodeType && o.cloneNode;
}

function setText(e, text) {
	dojox.data.dom.removeChildren(e);
	
	if (!isNode(text)) {
		text = document.createTextNode(text);
	}
	
	e.appendChild(text);
}


dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.FloatingLicenseUsageView", com.ibm.team.repository.web.ui.internal.admin.ItemList, {

	constructor: function(args){
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "FloatingLicenseUsageViewMessages");
		this.licenseId = args.licenseId;
	},
	
	postCreate: function() {
		com.ibm.team.repository.web.ui.internal.admin.server.FloatingLicenseUsageView.superclass.postCreate.apply(this);
		setText(this.heading, this.messages.heading);
		this._refresh();
	},
	
	_handleTypeListLoad: function(licenseId) {
		if (!licenseId) {
			// No floating licenses are installed in the server
			dojo.toggleClass(this.tableContents,"noDisplay",true);
			var descNode = document.createElement("span");
			var floatinglink = "<a href='#action=com.ibm.team.repository.admin.manageLicenses&focusTab=floating'>";
			var installFloatingMessage = dojo.string.substitute(this.messages.installFloatingLink, [ floatinglink, "</a>" ]);
			descNode.innerHTML = installFloatingMessage;
			this.updateDescription(descNode);
		} else {
			dojo.toggleClass(this.tableContents,"noDisplay",false);
			this.licenseId = licenseId;
			this._fetchAndDisplayLeaseInfo();
		}
	},
	
	_handleTypeChanged: function(licenseId) {
		document.location.href = "#action=com.ibm.team.repository.admin.floatingLicenseUsage&licenseId=" + licenseId;
		this.licenseId = licenseId;
		this._fetchAndDisplayLeaseInfo();
	},
	
	_refresh: function() {
		var description = document.createElement("div");
		var descritionText = document.createTextNode(this.messages.description);		
		description.appendChild(descritionText);
		
		licenseSelectDiv = document.createElement("div");
		licenseSelectDiv.style.margin = "1em 0";
		description.appendChild(licenseSelectDiv);
		
		this.updateDescription(description);

		var self = this;
		var onLoad = function(licenseId) {
			self._handleTypeListLoad(licenseId);
		}
		
		var onChange = function(licenseId) {
			self._handleTypeChanged(licenseId);
		}
		
		var onError = function(error) {
			self._handleActiveLeaseFetchError(error);
		} 

		var floatHelper = {
			yes: function(status) {
				var params = {licenseId: self.licenseId,
							  onLoad: onLoad,
							  onChange: onChange, 
							  onError: onError}	
				self.typeCombo = new FloatingLicenseTypeCombo(params);
				licenseSelectDiv.appendChild(self.typeCombo.domNode);
			},
			no: function(s) {
				setText(self.description, self.messages.notEnabled);  
			},
			error: function(err) {
				self._displayErrorMessage(self.messages.failedFloatCheck, err);
			}
		}
		
		LicenseClient.checkFloatingLicensesSupported(floatHelper.yes, floatHelper.no, floatHelper.error);
	},
	
	updateDescription: function(description) {
		// using the newItemButtonArea instead of "description" since we're adding <div> tags that shouldn't 
		// be appended to a <p> tag.  The name "newItemButtonArea" doesn't apply here... 
		// just needed another <div> without changing the template now.
		dojo.toggleClass(this.newItemButtonArea,"buttonArea",false);
		setText(this.newItemButtonArea, description);
	},

	
	_fetchAndDisplayLeaseInfo: function() {
		var self = this;
		var tempUrl = net.jazz.ajax._contextRoot + FLOATING_LICENSE_SERVICE_URL + "/"+ this.licenseId + FLOATING_LICENSE_SERVICE_URL_METHOD;
		var bindArgs= {
	        url: tempUrl,
	        content: { },
			handleAs: "json",
			headers: { "X-Jazz-CSRF-Prevent": dojo.cookie("JSESSIONID") },
			handle: dojo.hitch(this, function(response, ioArgs){
				if (response instanceof Error) {
					var errorObj= {
						status: ioArgs.xhr.status
					};
					// format detailed message (unescape URI-encoding formatting, remove leading dashes, and trim whitespace)
					errorObj.message= dojo.trim(unescape(ioArgs.xhr.statusText).replace(/\+/g, " ").replace(/^(-*)/g, ""));
					self._handleActiveLeaseFetchError(errorObj);
				}
				else {
					self._displayActiveLeaseInfo(response);
				}
	        })
	    };
	    dojo.xhrGet(bindArgs);
	},
	

	
	_displayActiveLeaseInfo: function(info) {
		dojox.data.dom.removeChildren(this.tbody);
		
		this.setHeaderRow([this.messages.server, this.messages.userId, 
			this.messages.operation, this.messages.timeout]);
		if (info.used) {
			var self = this;
			if (info.checkouts) {
				// Sort the leases so that the ones expiring soonest
				// are put first
				info.checkouts.sort(function(lease1, lease2){
					var date1 = dojo.date.stamp.fromISOString(lease1.endTime);
					var date2 = dojo.date.stamp.fromISOString(lease2.endTime);
					
					return date1.getTime() - date2.getTime();
				});
				
				
				// Add the sorted leases to the table
				dojo.forEach(info.checkouts, function(lease){
					var timeLeft = util.convertDateStringToRelativeTimeString(lease.endTime);
					var userIdData = lease.userId;
					var server = lease.server;
					
					var row = self.addRow(null, [lease.server, userIdData, lease.operationId, timeLeft]);
					dojo.toggleClass(row.parentNode,"userTable",true);
				});
			}
		} else {
			var tr = document.createElement("tr");
			var td = document.createElement("td");
			td.appendChild(document.createTextNode(this.messages.noIssued));
			td.colSpan = 4;
			tr.appendChild(td);
			tr.onclick = function (e) {
				dojo.stopEvent(e);
			}
			this.tbody.appendChild(tr);
		}
		
	},
	
	_handleActiveLeaseFetchError: function(errorObj) {
		this._displayErrorMessage(this.messages.serviceError, errorObj);
	},	

	_displayErrorMessage: function(message, errorObj) {
		if (errorObj) {
			var detailedReason = errorObj.status === 0 ? this.messages.unreachable : errorObj.message;
			message += dojo.string.substitute(this.messages.detailed, [ detailedReason ]); ;
		}
		this.displayErrorMessage(message, false);
	}


});

})();